<?php
/*
Plugin Name: RSS Reader by EasyToolsWP
Description: A custom RSS feed reader with dashboard widget and shortcode.
Version: 1.1
Author: EasyToolsWP
*/

// Register Custom Post Type for RSS Feeds
function etwp_register_rss_feed_post_type() {
    $labels = array(
        'name'                  => __('RSS Feeds'),
        'singular_name'         => __('RSS Feed'),
        'add_new'               => __('Add New Feed'),
        'add_new_item'          => __('Add New Feed'),
        'edit_item'             => __('Edit Feed'),
        'new_item'              => __('New Feed'),
        'view_item'             => __('View Feed'),
        'search_items'          => __('Search Feeds'),
        'not_found'             => __('No feeds found'),
        'not_found_in_trash'    => __('No feeds found in Trash'),
        'parent_item_colon'     => '',
        'menu_name'             => __('RSS Feeds')
    );

    register_post_type('etwp_rss_feed',
        array(
            'labels' => $labels,
            'public' => true,
            'has_archive' => true,
            'supports' => array('title'),
            'show_in_menu' => true,
            'show_in_admin_bar' => true,
            'menu_position' => 5,
			'menu_icon' => 'dashicons-rss', // Change the icon to RSS
        )
    );
}
add_action('init', 'etwp_register_rss_feed_post_type');

// Add meta box for RSS Feed URL and Dashboard Display Checkbox
function etwp_add_meta_boxes() {
    add_meta_box(
        'etwp_rss_feed_meta_box',
        'RSS Feed Details',
        'etwp_render_meta_box_content',
        'etwp_rss_feed',
        'normal',
        'high'
    );
}
add_action('add_meta_boxes', 'etwp_add_meta_boxes');

function etwp_render_meta_box_content($post) {
    $feed_url = get_post_meta($post->ID, '_etwp_feed_url', true);
    $show_on_dashboard = get_post_meta($post->ID, '_etwp_show_on_dashboard', true);
    $num_items = get_post_meta($post->ID, '_etwp_num_items', true);
    $open_in_new_tab = get_post_meta($post->ID, '_etwp_open_in_new_tab', true);
    ?>
    <label for="etwp_feed_url">Feed URL:</label>
    <input type="text" id="etwp_feed_url" name="etwp_feed_url" value="<?php echo esc_attr($feed_url); ?>" size="50" />
    <br><br>
    <label for="etwp_show_on_dashboard">
        <input type="checkbox" id="etwp_show_on_dashboard" name="etwp_show_on_dashboard" <?php checked($show_on_dashboard, 'on'); ?> />
        Show on Dashboard
    </label>
    <br><br>
    <label for="etwp_num_items">Number of Items to Show:</label>
    <input type="number" id="etwp_num_items" name="etwp_num_items" value="<?php echo esc_attr($num_items); ?>" min="1" />
    <br><br>
    <label for="etwp_open_in_new_tab">
        <input type="checkbox" id="etwp_open_in_new_tab" name="etwp_open_in_new_tab" <?php checked($open_in_new_tab, 'on'); ?> />
        Open links in new tab
    </label>
<style>#minor-publishing-actions, #edit-slug-box{display:none;}</style>
    <?php
}

function etwp_save_meta_box_data($post_id) {
    if (array_key_exists('etwp_feed_url', $_POST)) {
        update_post_meta($post_id, '_etwp_feed_url', sanitize_text_field($_POST['etwp_feed_url']));
    }
    if (array_key_exists('etwp_show_on_dashboard', $_POST)) {
        update_post_meta($post_id, '_etwp_show_on_dashboard', 'on');
    } else {
        update_post_meta($post_id, '_etwp_show_on_dashboard', 'off');
    }
    if (array_key_exists('etwp_num_items', $_POST)) {
        update_post_meta($post_id, '_etwp_num_items', intval($_POST['etwp_num_items']));
    }
    if (array_key_exists('etwp_open_in_new_tab', $_POST)) {
        update_post_meta($post_id, '_etwp_open_in_new_tab', 'on');
    } else {
        update_post_meta($post_id, '_etwp_open_in_new_tab', 'off');
    }
}
add_action('save_post', 'etwp_save_meta_box_data');

// Add Dashboard Widgets
function etwp_dashboard_widgets() {
    $args = array(
        'post_type' => 'etwp_rss_feed',
        'meta_key' => '_etwp_show_on_dashboard',
        'meta_value' => 'on',
        'posts_per_page' => -1,
    );

    $query = new WP_Query($args);
    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            $feed_url = get_post_meta(get_the_ID(), '_etwp_feed_url', true);
            $open_in_new_tab = get_post_meta(get_the_ID(), '_etwp_open_in_new_tab', true);
            $num_items = intval(get_post_meta(get_the_ID(), '_etwp_num_items', true)); // Make sure it's an integer

            if ($feed_url) {
                wp_add_dashboard_widget(
                    'etwp_dashboard_widget_' . get_the_ID(),
                    get_the_title(),
                    function() use ($feed_url, $open_in_new_tab, $num_items) {
                        $response = wp_remote_get($feed_url);
                        if (is_wp_error($response)) {
                            echo '<p>Unable to fetch RSS feed.</p>';
                            return;
                        }

                        $body = wp_remote_retrieve_body($response);

                        // Attempt to parse XML
                        libxml_use_internal_errors(true);
                        $xml = simplexml_load_string($body, 'SimpleXMLElement', LIBXML_NOCDATA);
                        $errors = libxml_get_errors();
                        libxml_clear_errors();

                        if ($xml === false) {
                            echo '<p>Empty Feed</p>';
                            return;
                        }

                        // Check if XML object is valid and contains expected elements
                        if (isset($xml->channel) && isset($xml->channel->item) && count($xml->channel->item) > 0) {
                            // Convert SimpleXMLElement to an array
                            $items_array = [];
                            foreach ($xml->channel->item as $item) {
                                $items_array[] = [
                                    'title' => (string)$item->title,
                                    'link' => (string)$item->link
                                ];
                            }
                            $items_to_show = min($num_items, count($items_array));
                            echo '<ul>';
                            foreach (array_slice($items_array, 0, $items_to_show) as $item) {
                                $target = $open_in_new_tab ? ' target="_blank"' : '';
                                echo '<li><a href="' . esc_url($item['link']) . '"' . $target . '>' . esc_html($item['title']) . '</a></li>';
                            }
                            echo '</ul>';
                        } else {
                            echo '<p>Empty Feed</p>';
                        }
                    }
                );
            }
        }
        wp_reset_postdata();
    }
}
add_action('wp_dashboard_setup', 'etwp_dashboard_widgets');

// Register Shortcode
function etwp_rss_feed_shortcode($atts) {
    $atts = shortcode_atts(
        array(
            'id' => 0,
            'num_items' => 5,
        ),
        $atts,
        'etwp_rss_feed'
    );

    $feed = get_post($atts['id']);
    if (!$feed) return 'Feed not found.';

    $feed_url = get_post_meta($feed->ID, '_etwp_feed_url', true);
    $open_in_new_tab = get_post_meta($feed->ID, '_etwp_open_in_new_tab', true);
    $num_items = $atts['num_items'] ?: get_post_meta($feed->ID, '_etwp_num_items', true) ?: 5;

    if (empty($feed_url)) {
        return 'Feed URL is empty.';
    }

    $rss = fetch_feed($feed_url);
    if (is_wp_error($rss)) {
        return 'Empty Feed';
    }

    if ($rss->get_item_quantity() == 0) {
        return 'Empty Feed';
    }

    $items = $rss->get_items(0, $num_items);
    if (!empty($items)) {
        $output = '<ul>';
        foreach ($items as $item) {
            $target = $open_in_new_tab ? ' target="_blank"' : '';
            $output .= '<li><a href="' . esc_url($item->get_permalink()) . '"' . $target . '>' . esc_html($item->get_title()) . '</a></li>';
        }
        $output .= '</ul>';
        return $output;
    } else {
        return 'Empty Feed';
    }
}
add_shortcode('etwp_rss_feed', 'etwp_rss_feed_shortcode');

// Direct RSS feed shortcode without requiring post creation
function etwp_rss_feed_direct_shortcode($atts) {
    $atts = shortcode_atts(
        array(
            'url' => '',
            'num_items' => 5,
        ),
        $atts,
        'etwp_rss_feed_direct'
    );

    $feed_url = $atts['url'];
    $num_items = intval($atts['num_items']);

    if (empty($feed_url)) {
        return 'Feed URL is empty.';
    }

    $rss = fetch_feed($feed_url);
    if (is_wp_error($rss)) {
        return 'Empty Feed';
    }

    if ($rss->get_item_quantity() == 0) {
        return 'Empty Feed';
    }

    $items = $rss->get_items(0, $num_items);
    if (!empty($items)) {
        $output = '<ul>';
        foreach ($items as $item) {
            $output .= '<li><a href="' . esc_url($item->get_permalink()) . '">' . esc_html($item->get_title()) . '</a></li>';
        }
        $output .= '</ul>';
        return $output;
    } else {
        return 'Empty Feed';
    }
}
add_shortcode('etwp_rss_feed_direct', 'etwp_rss_feed_direct_shortcode');

// Add custom columns to the RSS Feed admin list
function etwp_add_custom_columns($columns) {
    $columns['shortcode'] = 'Shortcode';
    $columns['feed_url'] = 'Feed URL';
    $columns['num_items'] = 'Items';
    $columns['target'] = 'Target';
    return $columns;
}
add_filter('manage_etwp_rss_feed_posts_columns', 'etwp_add_custom_columns');

function etwp_custom_column_content($column, $post_id) {
    switch ($column) {
        case 'shortcode':
            echo '[etwp_rss_feed id="' . $post_id . '"]';
            break;
        case 'feed_url':
            $feed_url = get_post_meta($post_id, '_etwp_feed_url', true);
            echo esc_html($feed_url);
            break;
        case 'num_items':
            $num_items = get_post_meta($post_id, '_etwp_num_items', true);
            echo esc_html($num_items);
            break;
        case 'target':
            $open_in_new_tab = get_post_meta($post_id, '_etwp_open_in_new_tab', true);
            echo $open_in_new_tab === 'on' ? 'Yes' : 'No';
            break;
    }
}
add_action('manage_etwp_rss_feed_posts_custom_column', 'etwp_custom_column_content', 10, 2);

// Enqueue custom admin styles
function etwp_enqueue_admin_styles() {
    echo '<style>
        .inside .slug { display: none; }
    </style>';
}
add_action('admin_head', 'etwp_enqueue_admin_styles');
